﻿using Microsoft.EntityFrameworkCore;
using SoftwareInstallationContracts.BindingModels;
using SoftwareInstallationContracts.SearchModels;
using SoftwareInstallationContracts.StoragesContracts;
using SoftwareInstallationContracts.ViewModels;
using SoftwareInstallationDatabaseImplement.Models;

namespace SoftwareInstallationDatabaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new SoftwareInstallationDatabase();
            var element = context.Orders
                .Include(x => x.Implementer)
                .Include(x => x.Client)
                .FirstOrDefault(x => x.Id == model.Id);
            if (element != null)
            {
                context.Orders.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }

        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            using var context = new SoftwareInstallationDatabase();
            if (!model.Id.HasValue)
            {
                return null;
            }
			return context.Orders
					 .Include(x => x.Client)
					 .Include(x => x.Implementer)
					 .FirstOrDefault(x =>
						 (model.Statusses == null || model.Statusses != null && model.Statusses.Contains(x.Status)) &&
						 model.ImplementerId.HasValue && x.ImplementerId == model.ImplementerId ||
						 model.Id.HasValue && x.Id == model.Id
                         )?.GetViewModel;
		}

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
			if (model.Id.HasValue)
			{
				var result = GetElement(model);
				return result != null ? new() { result } : new();
			}

			using var context = new SoftwareInstallationDatabase();
			IQueryable<Order>? queryWhere = null;

			if (model.DateFrom.HasValue && model.DateTo.HasValue) // если не ищем по айдишнику, значит ищем по диапазону дат
			{
				queryWhere = context.Orders.Where(x => model.DateFrom <= x.DateCreate.Date && x.DateCreate.Date <= model.DateTo);
			}
			else if (model.Statusses != null)
			{
				queryWhere = context.Orders.Where(x => model.Statusses.Contains(x.Status));
			}
			else if (model.ClientId.HasValue)
			{
				queryWhere = context.Orders.Where(x => x.ClientId == model.ClientId);
			}
			else
			{
				return new();
			}
			return queryWhere
					.Include(x => x.Client)
					.Include(x => x.Implementer)
					.Select(x => x.GetViewModel)
					.ToList();
		}

        public List<OrderViewModel> GetFullList()
        {
            using var context = new SoftwareInstallationDatabase();
            return context.Orders
				.Include(x => x.Client)
				.Include(x => x.Implementer)
				.Include(x => x.Package)
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            using var context = new SoftwareInstallationDatabase();
            context.Orders.Add(newOrder);
            context.SaveChanges();
            return newOrder.GetViewModel;
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new SoftwareInstallationDatabase();
            var order = context.Orders
                .Include(x => x.Client)
				.Include(x => x.Implementer)
				.Include(x => x.Package)
                .FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }
            order.Update(model);
            context.SaveChanges();
            return order.GetViewModel;
        }
    }
}
