using SoftwareInstallationContracts.BusinessLogicsContracts;
using SoftwareInstallationContracts.StoragesContracts;
using SoftwareInstallationDatabaseImplement.Implements;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog.Extensions.Logging;
using SoftwareInstallationBusinessLogic.BusinessLogics;
using SoftwareInstallationBusinessLogic.OfficePackage.Implements;
using SoftwareInstallationBusinessLogic.OfficePackage;
using SoftwareInstallationBusinessLogic;

namespace SoftwareInstallationView
{
    internal static class Program
    {
        private static ServiceProvider? _serviceProvider;
        public static ServiceProvider? ServiceProvider => _serviceProvider;
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // To customize application configuration such as set high DPI settings or default font,
            // see https://aka.ms/applicationconfiguration.
            ApplicationConfiguration.Initialize();
            var services = new ServiceCollection();
            ConfigureServices(services);
            _serviceProvider = services.BuildServiceProvider();
            Application.Run(_serviceProvider.GetRequiredService<FormMain>());
        }
        private static void ConfigureServices(ServiceCollection services)
        {
            services.AddLogging(option =>
            {
                option.SetMinimumLevel(LogLevel.Information);
                option.AddNLog("nlog.config");
            });
            services.AddTransient<IComponentStorage, ComponentStorage>();
            services.AddTransient<IOrderStorage, OrderStorage>();
            services.AddTransient<IPackageStorage, PackageStorage>();
			services.AddTransient<IImplementerStorage, ImplementerStorage>();
			services.AddTransient<IImplementerLogic, ImplementerLogic>();
			services.AddTransient<IComponentLogic, ComponentLogic>();
            services.AddTransient<IOrderLogic, OrderLogic>();
            services.AddTransient<IPackageLogic, PackageLogic>();
            services.AddTransient<IClientStorage, ClientStorage>();
            services.AddTransient<IClientLogic, ClientLogic>();
            services.AddTransient<IReportLogic, ReportLogic>();
			services.AddTransient<IWorkProcess, WorkModeling>();

			services.AddTransient<AbstractSaveToExcel, SaveToExcel>();
            services.AddTransient<AbstractSaveToWord, SaveToWord>();
            services.AddTransient<AbstractSaveToPdf, SaveToPdf>();

            services.AddTransient<FormMain>();
            services.AddTransient<FormComponent>();
            services.AddTransient<FormComponents>();
            services.AddTransient<FormCreateOrder>();
            services.AddTransient<FormPackage>();
            services.AddTransient<FormPackageComponent>();
            services.AddTransient<FormPackages>();
            services.AddTransient<FormReportOrders>();
            services.AddTransient<FormReportPackageComponents>();
            services.AddTransient<FormViewClients>();
			services.AddTransient<FormViewImplementers>();
			services.AddTransient<FormImplementer>();
		}
    }
}